import re
import PIL
import cStringIO
import StringIO

from django.core.files.uploadedfile import InMemoryUploadedFile
from django.utils.datastructures import MultiValueDict
from django.forms.widgets import ClearableFileInput


class ImageFileInput(ClearableFileInput):

    template = """\
    <div class='ifm js-fileapi-wrapper' data-isfile='{isfile}' data-istemp='{istemp}'
        style='height:{height}px; width:{width}px' data-aspectratio={aspectratio} data-name='{name}64' {data} >
    {clear}<div class='ifmpreview'>{img}</div>
    <input type='hidden' name='{name}64' />{rendered}</div>"""

    def __init__(self, *args, **kwargs):
        """File inpur with crop and resize client side
        :arg minh: minimin height (px)
        :arg minw: minimin width (px)
        :arg maxh: maximum height (px)
        :arg maxw: maximum width (px)
        :arg aspectratio: aspect ratio (default minw/minh). -1 to disable any constraint
        :arg prev-height: height of the preview image (default 150px)
        :arg icon: name of the font-awesome icon to use as a placeholder
        """
        self.base_attrs = {'prev-height': 150}
        self.base_attrs.update(kwargs)
        super(ImageFileInput, self).__init__(*args, **kwargs)

    def convert(self, base64encoded, name):
        image_data = re.sub('^data:image/.+;base64,', '', base64encoded).decode('base64')
        image = PIL.Image.open(cStringIO.StringIO(image_data))
        thumb_io = StringIO.StringIO()
        image.save(thumb_io, format='JPEG')
        upload = InMemoryUploadedFile(thumb_io, None, name + '.jpg', 'image/jpeg',
                                      thumb_io.len, None)
        upload.seek(0)
        return upload

    def value_from_datadict(self, data, files, name):
        base64encoded = data.get(name + "64", False)
        if base64encoded:
            upload = self.convert(base64encoded, name)
            files64 = MultiValueDict({})
            files64[name] = upload
            return super(ImageFileInput, self).value_from_datadict(data, files64, name)
        return super(ImageFileInput, self).value_from_datadict(data, files, name)

    class Media:
        css = {'all': ('imagefileinput/Jcrop/jquery.Jcrop.min.css',
                       'imagefileinput/imagefileinput.css')}
        js = (
            'imagefileinput/Jcrop/jquery.Jcrop.min.js',
            'imagefileinput/imagefileinput.js',
            'imagefileinput/FileAPI/FileAPI.min.js',
            'imagefileinput/FileAPI/FileAPI.exif.js',
        )

    def render(self, name, value, attrs=None):
        attrs.update({"accept": "image/*"})
        rendered = super(ImageFileInput, self).render(name, None, attrs)
        self.base_attrs.update(self.build_attrs())
        height = self.base_attrs.get('prev-height', 100)
        width = self.base_attrs.get('prev-width', 100)
        icon = self.base_attrs.get('icon', 'picture-o')
        autoaspectratio = self.base_attrs.get('minw', 100) / float(self.base_attrs.get('minh', 100))
        aspectratio = self.base_attrs.get('aspectratio', autoaspectratio)

        accept = "image/*"
        data = ""
        for key, val in self.base_attrs.iteritems():
            data += " data-{key}={val}".format(**locals())
        try:
            img = "<img class='ifmimgorig' height='{height}' src='{url}'>".format(
                height=height, url=value.url) if value else ""
        except:
            img = ""

        isfile = True if value and hasattr(value, 'url') else False
        istemp = True if value and not hasattr(value, 'url') else False
        img += "<div class='ifmimgholder holder-{name}'><i class='fa fa-{icon}'></i></div>".format(
            name=name, icon=icon)

        if not self.is_required and isfile:
            clear = "<input style='display:none' id='{id}' name='{name}' type='checkbox'>".format(
                id=self.clear_checkbox_id(name), name=self.clear_checkbox_name(name))
            clear += '<button  class="ifmimgbtnorig btn"><i class="fa fa-close"></i></button>'
            clear += ('<button style="display:none"  class="ifmimgbtnorigrestore btn">'
                      '<i class="fa fa-undo"></i></button>')
        else:
            clear = ""

        txt = self.template.format(**locals())
        return txt
